"use strict";
/*
 * Copyright (c) 2021, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
const os_1 = require("os");
const core_1 = require("@oclif/core");
const core_2 = require("@salesforce/core");
const kit_1 = require("@salesforce/kit");
const ts_types_1 = require("@salesforce/ts-types");
const types_1 = require("@sf/sdr/lib/src/client/types");
const sf_plugins_core_1 = require("@salesforce/sf-plugins-core");
const orgs_1 = require("../../utils/orgs");
const componentSetBuilder_1 = require("../../utils/componentSetBuilder");
const output_1 = require("../../utils/output");
const testLevel_1 = require("../../utils/testLevel");
const progressBar_1 = require("../../utils/progressBar");
const config_1 = require("../../utils/config");
const requiredFlagValidator_1 = require("../../utils/requiredFlagValidator");
core_2.Messages.importMessagesDirectory(__dirname);
const messages = core_2.Messages.loadMessages('@salesforce/plugin-deploy-retrieve-metadata', 'deploy.metadata');
// One of these flags must be specified for a valid deploy.
const requiredFlags = ['manifest', 'metadata', 'source-dir'];
class DeployMetadata extends sf_plugins_core_1.SfCommand {
    async run() {
        const flags = (await this.parse(DeployMetadata)).flags;
        const testLevel = flags['test-level'];
        requiredFlagValidator_1.validateOneOfCommandFlags(requiredFlags, flags);
        const componentSet = await componentSetBuilder_1.ComponentSetBuilder.build({
            sourcepath: flags['source-dir'],
            manifest: (flags.manifest && {
                manifestPath: flags.manifest,
                directoryPaths: await orgs_1.getPackageDirs(),
            }),
            metadata: flags.metadata && {
                metadataEntries: flags.metadata,
                directoryPaths: await orgs_1.getPackageDirs(),
            },
        });
        const targetOrg = await orgs_1.resolveTargetOrg(flags['target-org']);
        this.log(`${os_1.EOL}${messages.getMessage('deploy.metadata.api', [targetOrg, config_1.resolveRestDeploy()])}${os_1.EOL}`);
        const deploy = await componentSet.deploy({
            usernameOrConnection: targetOrg,
            apiOptions: { testLevel },
        });
        if (!this.jsonEnabled()) {
            new progressBar_1.DeployProgress(deploy).start();
        }
        const result = await deploy.pollStatus(500, kit_1.Duration.minutes(flags.wait).seconds);
        this.setExitCode(result);
        if (!this.jsonEnabled()) {
            output_1.displaySuccesses(result);
            output_1.displayFailures(result);
            output_1.displayTestResults(result, testLevel);
        }
        return {
            files: output_1.asRelativePaths((result === null || result === void 0 ? void 0 : result.getFileResponses()) || []),
            tests: this.getTestResults(result),
        };
    }
    setExitCode(result) {
        const status = ts_types_1.getString(result, 'response.status');
        if (status !== types_1.RequestStatus.Succeeded) {
            process.exitCode = 1;
        }
    }
    getTestResults(result) {
        const passing = ts_types_1.get(result, 'response.numberTestsCompleted', 0);
        const failing = ts_types_1.get(result, 'response.numberTestErrors', 0);
        const total = ts_types_1.get(result, 'response.numberTestsTotal', 0);
        const testResults = { passing, failing, total };
        const time = ts_types_1.get(result, 'response.details.runTestResult.totalTime', 0);
        return time ? { ...testResults, time } : testResults;
    }
}
exports.default = DeployMetadata;
DeployMetadata.description = messages.getMessage('description');
DeployMetadata.summary = messages.getMessage('summary');
DeployMetadata.examples = messages.getMessages('examples');
DeployMetadata.flags = {
    metadata: core_1.Flags.string({
        char: 'm',
        summary: messages.getMessage('flags.metadata.summary'),
        multiple: true,
        exclusive: ['manifest', 'source-dir'],
    }),
    manifest: core_1.Flags.string({
        char: 'x',
        description: messages.getMessage('flags.manifest.description'),
        summary: messages.getMessage('flags.manifest.summary'),
        exclusive: ['metadata', 'source-dir'],
    }),
    'source-dir': core_1.Flags.string({
        char: 'd',
        description: messages.getMessage('flags.source-dir.description'),
        summary: messages.getMessage('flags.source-dir.summary'),
        multiple: true,
        exclusive: ['manifest', 'metadata'],
    }),
    'target-org': core_1.Flags.string({
        char: 'o',
        description: messages.getMessage('flags.target-org.description'),
        summary: messages.getMessage('flags.target-org.summary'),
    }),
    'test-level': core_1.Flags.string({
        char: 'l',
        description: messages.getMessage('flags.test-level.description'),
        summary: messages.getMessage('flags.test-level.summary'),
        options: Object.values(testLevel_1.TestLevel),
        default: testLevel_1.TestLevel.NoTestRun,
    }),
    wait: core_1.Flags.integer({
        char: 'w',
        summary: messages.getMessage('flags.wait.summary'),
        description: messages.getMessage('flags.wait.description'),
        default: 33,
    }),
};
DeployMetadata.configurationVariablesSection = sf_plugins_core_1.toHelpSection('CONFIGURATION VARIABLES', core_2.OrgConfigProperties.TARGET_ORG, core_2.SfdxPropertyKeys.API_VERSION);
DeployMetadata.envVariablesSection = sf_plugins_core_1.toHelpSection('ENVIRONMENT VARIABLES', core_2.EnvironmentVariable.SF_TARGET_ORG, core_2.EnvironmentVariable.SFDX_DEFAULTUSERNAME, core_2.EnvironmentVariable.SFDX_USE_PROGRESS_BAR);
//# sourceMappingURL=metadata.js.map